/***********************************************************
 * Hajime no Ippo --- file : hjime.cpp
 ***********************************************************/
#include <ode/ode.h>
#include <drawstuff/drawstuff.h>

#define MAX_CONTACTS 4
static dWorldID world;
static dSpaceID space;
static dJointGroupID contactgroup;

static dBodyID b_sphere;
static dGeomID g_sphere;
static dReal  radius = 0.5;

#ifdef dDOUBLE
  #define dsDrawSphere dsDrawSphereD
  #define dsDrawBox dsDrawBoxD
#endif

static void nearCallback( void *data, dGeomID o1, dGeomID o2 )
{
  dBodyID b1 = dGeomGetBody( o1 );
  dBodyID b2 = dGeomGetBody( o2 );

  if ( b1 && b2 && dAreConnectedExcluding( b1, b2, dJointTypeContact ) )
    return;

  dContact contact[MAX_CONTACTS];
  for ( int i=0; i<MAX_CONTACTS; i++ )
  {
    //contact[i].surface.mode = dContactBounce | dContactSoftERP | dContactSoftCFM;
    contact[i].surface.mode = dContactBounce;
    contact[i].surface.mu = dInfinity;
    contact[i].surface.bounce = 0.3;
    //contact[i].surface.soft_erp = 0.2;
    //contact[i].surface.soft_cfm = 0.00001;
  }

  int numc = dCollide( o1, o2, MAX_CONTACTS, &contact[0].geom, sizeof( dContact ) ); 
  if ( numc > 0 )
  {
    for ( int i=0; i<numc; i++ )
    {
      dJointID c = dJointCreateContact( world, contactgroup, contact+i );
      dJointAttach( c, b1, b2 );
    }
  }
}

// start simulation - set viewpoint
static void start()
{
  static float xyz[3] = { 0.0f, 7.0f, 5.0f };
  static float hpr[3] = { -90.f, -15.f, 0.f };

  dsSetViewpoint( xyz, hpr );
}

// simulation loop
static void simLoop( int pause )
{
  dSpaceCollide( space, 0, &nearCallback );

  if (!pause)
  {
    dWorldStep( world, 0.005 );
  }

  dJointGroupEmpty( contactgroup );

  dsSetColorAlpha( 0.0, 0.0, 1.0, 0.5 );
  dsDrawSphere( dBodyGetPosition( b_sphere ), dBodyGetRotation( b_sphere ), 0.1 ); // $B=E?4I=<((B
  dsSetColorAlpha( 1.0, 1.0, 0.0, 0.5 );
  dsDrawSphere( dGeomGetPosition( g_sphere ), dGeomGetRotation( g_sphere ), radius ); // $B5eBNI=<((B
}


int main( int argc, char* argv[] )
{
  // setup pointers to drawstuff callback functions
  dsFunctions fn;
  fn.version = DS_VERSION;
  fn.start   = &start;
  fn.step    = &simLoop;
  fn.command = 0;
  fn.stop    = 0;
  fn.path_to_textures = "../../drawstuff/textures";

  // create world
  world = dWorldCreate();
  space = dHashSpaceCreate( 0 );
  contactgroup = dJointGroupCreate( 0 );
  dCreatePlane( space, 0, 0, 1, 0 );
  dWorldSetGravity( world, 0.0, 0.0, -9.8 );
  dWorldSetERP( world, 0.8 );
  dWorldSetCFM( world, 0.00001 );

  // create Sphere
  dReal pos[3] = { 0.0, 0.0, 5.0 }; // $BJ*BN0LCV(B($B@dBPCM(B)
  dReal g_pos[3] = { 0.1, 0.0, 0.0 }; // $B=E?40LCV(B($BAjBPCM(B)
  b_sphere = dBodyCreate( world ); // set dBodyID
  dBodySetPosition( b_sphere, pos[0]+g_pos[0], pos[1]+g_pos[1], pos[2]+g_pos[2] );
  dMass sphere_mass;
  sphere_mass.setSphere( 1.0, radius );
  dBodySetMass( b_sphere, &sphere_mass ); // $B<ANL@_Dj(B

  g_sphere = dCreateSphere( space, radius ); // set dGeomID
  //dGeomSetPosition( g_sphere, pos[0], pos[1], pos[2] );
  dGeomSetBody( g_sphere, b_sphere );
  dGeomSetOffsetPosition( g_sphere, -g_pos[0], -g_pos[1], -g_pos[2] ); // $B=E?40LCVJQ99(B

  // starting simulation
  dsSimulationLoop( argc, argv, 320, 240, &fn );

  dJointGroupDestroy( contactgroup );
  dSpaceDestroy( space );
  dWorldDestroy( world );
  return 0;
}

